import numpy as np
import matplotlib.pyplot as plt
from matplotlib.path import Path as PATH
from matplotlib.patches import Polygon
import matplotlib.patheffects as pe
import pandas as pd
import os,sys
import itertools
from Thermobar.core import *
from Thermobar.geotherm import *
from Thermobar.garnet import *
from Thermobar.garnet_class import *

def plot_garnet_geotherm(gt_comps, T_Ni, P_Cr, BDL_T, SHF_low, SHF_high, max_depth, SHF_chosen = None, temp_unit = 'Celsius', plot_type = 'show', filename_save = None):

	'''
	A function to plot the CaO CR2O3 based garnet xenocryst classifications
	and to determine the base of the depleted lithosphere BDL_T.

	###Parameters###
	gt_comps: garnet composition dataframe imported from core function

	BDL_T: is Temperature at the base of the depleted lithosphere in Celsius

	T_Ni: Temperature output taken from one of the garnet thermometry methods in
	Kelvin.

	P_Cr: Cr2O3 based pressure estimate of the garnet xenocrysts in KBar

	SHF_low: Surface heat flow value for the lower end of the geotherms to be plotted in mW/m^2

	SHF_high: Surface heat flow value for the higher end of the geotherms to be plotted in mW/m^2

	max_depth: maximum depth for calculations and plotting

	SHF_chosen: Surface heat flow value for the chosen geotherm to be plotted in mW/m^2

	temp_unit: temperature unit 'Celsius' or 'Kelvin'

	plot_type: parameter to decide whether to 'show' the plot or 'save' it as a png file.

	filename_save: str parameter to determine the filename to be saved. If none entered it
	defaults to a certain name.

	'''

	geotherms = []
	pressures = []

	for i in range(SHF_low, SHF_high):

		T, depth, p, depth_intercepts = calculate_hasterok2011_geotherm(SHF = i, BDL_T = BDL_T + 273, T_0 = 0, max_depth = max_depth, moho = 38, kinked = True, adiabat = False)
		if temp_unit == 'Celsius':
			geotherms.append(T-273.0)
		pressures.append(p)


	geotherms = np.array(geotherms)
	pressures = np.array(pressures)

	P_Cr = P_Cr / 10.0

	d_g_p = [35,37.5,40,42.5,45,47.5,50,52.5,55,57.5,60]

	if temp_unit == 'Celsius':
		T_Ni = T_Ni - 273.0
		d_g_t = [600,700,800,900,1000,1100,1200,1300,1400,1500,1600]
	else:
		d_g_t = np.array([600,700,800,900,1000,1100,1200,1300,1400,1500,1600]) + 273

	#Checking if Y_Gt exist at all

	try:
		gt_comps['Y_Gt']
		trace_exist = True
	except KeyError:
		trace_exist = False

	if trace_exist == True:

		for i in range(0,len(gt_comps)):

			if pd.isna(gt_comps['Y_Gt'][i]) == True:

				print('WARNING')
				print('There is Y_Gt missing in one of the samples. idx = ' + str(i))

	ca_cr_class = garnet_ca_cr_class_Griffin2002(gt_comps)

	lherz_p = []
	lherz_t = []
	lowcaharz_p = []
	lowcaharz_t = []
	harz_p = []
	harz_t = []
	low_cr_gt_p = []
	low_cr_gt_t = []
	wehrlite_p = []
	wehrlite_t = []
	unclass_p = []
	unclass_t = []

	for i in range(0,len(T_Ni)):
		if ca_cr_class[i] == 'Lherzolite':
			lherz_p.append(float(P_Cr[i]))
			lherz_t.append(T_Ni[i])
		elif ca_cr_class[i] == 'Low-Ca Harzburgite':
			lowcaharz_p.append(float(P_Cr[i]))
			lowcaharz_t.append(T_Ni[i])
		elif ca_cr_class[i] == 'Ca-Harzburgite':
			harz_p.append(float(P_Cr[i]))
			harz_t.append(T_Ni[i])
		elif ca_cr_class[i] == 'Low-Cr Peridotite':
			low_cr_gt_p.append(float(P_Cr[i]))
			low_cr_gt_t.append(T_Ni[i])
		elif ca_cr_class[i] == 'Wehrlite':
			wehrlite_p.append(float(P_Cr[i]))
			wehrlite_t.append(T_Ni[i])
		else:
			unclass_p.append(float(P_Cr[i]))
			unclass_t.append(T_Ni[i])

	fig3 = plt.figure(figsize = (5,7))
	if trace_exist == True:
		ax1 = plt.subplot(211)
	elif trace_exist == False:
		ax1 = plt.subplot(111)

	ax1.plot(d_g_t,np.array(d_g_p)/10.0,'-',color = '#a832a4')

	for i in range(0,len(geotherms)):
		ax1.plot(geotherms[i],pressures[i],'--', linewidth = 0.75)


	if len(lherz_t) != 0:
		ax1.plot(lherz_t,lherz_p,'*',color = '#339e51', label = 'Lherzolite',markersize = 1.25)
	if len(lowcaharz_t) != 0:
		ax1.plot(lowcaharz_t,lowcaharz_p,'s',color = '#8b1f8f', label = 'Low-Ca Harzburgite',markersize = 1.25)
	if len(harz_t) != 0:
		ax1.plot(harz_t,harz_p,'s',color = '#ef2fa1', label = 'Harzburgite',markersize = 1.25)
	if len(low_cr_gt_t) != 0:
		ax1.plot(low_cr_gt_t,low_cr_gt_p,'o',color = '#d9c1cb', label = 'Low-Cr Peridotite',markersize = 1.25)
	if len(wehrlite_t) != 0:
		ax1.plot(wehrlite_t,wehrlite_p,'^',color = '#b06f90', label = 'Wehrlite',markersize = 1.25)
	if len(unclass_t) != 0:
		ax1.plot(unclass_t,unclass_p,'x',color = 'k',alpha = 0.25, label = 'Unclassified',markersize = 1.25)

	ax1.axvline(BDL_T,linestyle = '--',color = 'k',linewidth = 2)

	if SHF_chosen != None:
		T_chosen, depth_chosen, p_chosen, depth_intercept = calculate_hasterok2011_geotherm(SHF = SHF_chosen, BDL_T = BDL_T+273, T_0 = 0, max_depth = max_depth, moho = 38, kinked = True, adiabat = False)
		ax1.plot(T_chosen-273.0,p_chosen,'--', color = 'k', linewidth = 2)

	ax1.set_xlim((600,1800))
	ax1.set_ylim((9,0))
	ax1.legend(fontsize = 7,framealpha = 1)
	ax1.set_ylabel('P (GPa)')

	if trace_exist == True:
		ax2 = plt.subplot(212)

	if trace_exist == True:
		lherz_t = []
		lherz_y = []
		lowcaharz_t = []
		lowcaharz_y = []
		harz_t = []
		harz_y = []
		low_cr_gt_t = []
		low_cr_gt_y = []
		wehrlite_t = []
		wehrlite_y = []
		unclass_t = []
		unclass_y = []

		for i in range(0,len(T_Ni)):
			if ca_cr_class[i] == 'Lherzolite':
				lherz_t.append(float(T_Ni[i]))
				lherz_y.append(gt_comps['Y_Gt'][i])
			elif ca_cr_class[i] == 'Low-Ca Harzburgite':
				lowcaharz_t.append(float(T_Ni[i]))
				lowcaharz_y.append(gt_comps['Y_Gt'][i])
			elif ca_cr_class[i] == 'Ca-Harzburgite':
				harz_t.append(float(T_Ni[i]))
				harz_y.append(gt_comps['Y_Gt'][i])
			elif ca_cr_class[i] == 'Low-Cr Peridotite':
				low_cr_gt_t.append(float(T_Ni[i]))
				low_cr_gt_y.append(gt_comps['Y_Gt'][i])
			elif ca_cr_class[i] == 'Wehrlite':
				wehrlite_t.append(float(T_Ni[i]))
				wehrlite_y.append(gt_comps['Y_Gt'][i])
			else:
				unclass_t.append(float(T_Ni[i]))
				unclass_y.append(gt_comps['Y_Gt'][i])

		if len(lherz_t) != 0:
			ax2.plot(lherz_t,lherz_y,'*',color = '#339e51', label = 'Lherzolite',markersize = 3)
		if len(lowcaharz_t) != 0:
			ax2.plot(lowcaharz_t,lowcaharz_y,'s',color = '#8b1f8f', label = 'Low-Ca Harzburgite',markersize = 3)
		if len(harz_t) != 0:
			ax2.plot(harz_t,harz_y,'s',color = '#ef2fa1', label = 'Harzburgite',markersize = 3)
		if len(low_cr_gt_t) != 0:
			ax2.plot(low_cr_gt_t,low_cr_gt_y,'s',color = '#d9c1cb', label = 'Low-Cr Peridotite',markersize = 3)
		if len(wehrlite_t) != 0:
			ax2.plot(wehrlite_t,wehrlite_y,'^',color = '#b06f90', label = 'Wehrlite',markersize = 3)
		if len(unclass_t) != 0:
			ax2.plot(unclass_t,unclass_y,'x',color = 'k',alpha = 0.25, label = 'Unclassified',markersize = 3)

		ax2.set_ylim(1e-1,1e2)
		ax2.set_xlim(600,1800)
		if temp_unit == 'Celsius':
			ax2.set_xlabel(r'$T_{Ni} \, C^{\circ}$')
		else:
			ax2.set_xlabel(r'$T_{Ni} \, K^{\circ}$')
		ax2.set_ylabel('Y (wt ppm)')
		ax2.set_yscale('log')
		ax2.axvline(BDL_T,linestyle = '--',color = 'k',linewidth = 2)

	if plot_type == 'show':
		plt.show()
	elif plot_type == 'save':
		if plot_path == None:
			save_str = os.path.join((os.getcwd(),'CA_CR_PLOT.png'))
			plt.savefig(save_str, dpi = 300.0)
		else:
			try:
				plt.savefig(filename_save, dpi = 300.0,bbox_inches = "tight")
			except ValueError:
				print('Not a proper format for the filename_save')

def plot_garnet_composition_section(gt_comps, depth_interval, min_section_depth, max_section_depth, T_Ni, P_Cr, BDL_T, SHF_low, SHF_high, SHF_chosen = None, temp_unit = 'Celsius', plot_type = 'show', filename_save = None):

	'''
	A function to plot a collection of the garnet xenocryst based classifications

	###Parameters###
	gt_comps: garnet composition dataframe imported from core function

	depth_interval: depth interval for the histogram sections in km

	min_section_depth: depth for the minimum depth to be plotted in km

	max_section_depth: depth for the maximum depth to be plotted in km

	T_Ni: Temperature output taken from one of the garnet thermometry methods in
	Kelvin.

	P_Cr: Cr2O3 based pressure estimate of the garnet xenocrysts in KBar

	BDL_T: is Temperature at the base of the depleted lithosphere in Celsius

	SHF_low: Surface heat flow value for the lower end of the geotherms to be plotted in mW/m^2

	SHF_high: Surface heat flow value for the higher end of the geotherms to be plotted in mW/m^2

	SHF_chosen: Surface heat flow value for the chosen geotherm to be plotted in mW/m^2

	temp_unit: temperature unit 'Celsius' or 'Kelvin'

	max_depth: maximum depth for geotherm calculations in km

	plot_type: parameter to decide whether to 'show' the plot or 'save' it as a png file.

	filename_save: str parameter to determine the filename to be saved. If none entered it
	defaults to a certain name.

	'''

	P_Cr = P_Cr / 10.0

	#Calculating range of geotherms to be plotted
	geotherms = []
	pressures = []
	depths = []

	for i in range(SHF_low, SHF_high):

		T, depth, p, depth_intercepts = calculate_hasterok2011_geotherm(SHF = i, BDL_T = BDL_T+273, T_0 = 0, max_depth = max_section_depth, moho = 38, kinked = True, adiabat = False)
		if temp_unit == 'Celsius':
			geotherms.append(T-273.0)
		pressures.append(p)
		depths.append(depth)

	geotherms = np.array(geotherms)
	pressures = np.array(pressures)
	depths = np.array(depths)

	d_g_p = [35,37.5,40,42.5,45,47.5,50,52.5,55,57.5,60] #diamond-graphite transition pressure in kBar

	if temp_unit == 'Celsius':
		T_Ni = T_Ni - 273.0
		d_g_t = [600,700,800,900,1000,1100,1200,1300,1400,1500,1600] #diamond graphite transition temp in Celsius
	else:
		d_g_t = np.array([600,700,800,900,1000,1100,1200,1300,1400,1500,1600]) + 273 #diamond graphite transition temp in Kelvin

	depth_fields = np.arange(min_section_depth, max_section_depth + depth_interval, depth_interval) * 1e3 #Creating depth ranges for the plot sections with the
	#interval inputted with min_section_depth, max_section_depth and depth_interval

	gt_calc = calculate_garnet_components(gt_comps = gt_comps)

	xMg = np.array(gt_calc['Mg_MgFeCa_Gt'])
	xCa = np.array(gt_calc['Ca_MgFeCa_Gt'])
	xFe = np.array(gt_calc['Fe_MgFeCa_Gt'])
	xAl = np.array(gt_calc['Al_AlCr_Gt'])
	xCr = np.array(gt_calc['Cr_AlCr_Gt'])

	carp_depleted_harz, carp_depleted_lherz, carp_depleted_metasomatised, carp_fertile_lherz, carp_melt_metas, carp_unclass, len_tot = garnet_CARP_class_Griffin2002(gt_comps = gt_comps) #determining CARP classes from garnet composition
	cacr_class = garnet_ca_cr_class_Griffin2002(gt_comps = gt_comps)
	g_class = garnet_class_Grutter2004(gt_comps = gt_comps)
	MG = calculate_ol_mg(gt_comps = gt_comps, T_Ni = T_Ni) #calculating olivine magnesium number from garnet composition
	AL = calculate_al2o3_whole_rock(gt_comps = gt_comps) #calculating Al2O3 whole rock from garnet compostion

	depth_normal = np.zeros(len(P_Cr))

	#Creating the chosen geotherm for the garnet
	if SHF_chosen != None:
		T_chosen, depth_chosen, p_chosen, depth_intercept = calculate_hasterok2011_geotherm(SHF = SHF_chosen, BDL_T = BDL_T + 273, T_0 = 0, max_depth = max_section_depth, moho = 38, kinked = True, adiabat = False)

		if temp_unit == 'Celsius':
			T_chosen = T_chosen - 273.0

		interp_T = np.arange(np.amin(T_chosen),np.amax(T_chosen),3)
		interp_depth = np.interp(interp_T, T_chosen, depth_chosen)

		BDL_depth = depth_chosen[depth_intercept]

		for i in range(0,len(P_Cr)):
			#for look that downprojects the P_Cr onto the selected
			#geotherö
			idx_fnd = (np.abs(interp_T-T_Ni[i])).argmin()
			depth_normal[i] = interp_depth[idx_fnd]

	def count_arrays(array,depth_array,depth_grid,index_depth):
		#local def to count arrays
		idx = 0

		for index in array:
			if (depth_array[index] < depth_grid[index_depth]) and (depth_array[index] >= depth_grid[index_depth-1]):
				idx = idx + 1

		return idx

	def pct_count(count,tot_num):
		#local def to convert
		try:
			pct = (float(count) / float(tot_num)) * 100.0
		except ZeroDivisionError:
			pct = 0
		return pct

	deplet_harz_pct_list = []
	deplet_lherz_pct_list = []
	deplet_metas_pct_list = []
	fertile_pct_list = []
	melt_pct_metas = []
	l10a_over_fert = []
	unclass_pct = []
	total_list = []
	total_classified = []

	for j in range(1,len(depth_fields)):

		tot_num_depth = 0
		carp_deplet_harz_count = count_arrays(carp_depleted_harz,depth_normal,depth_fields,j)
		carp_deplet_lherz_count = count_arrays(carp_depleted_lherz,depth_normal,depth_fields,j)
		carp_depleted_metasomatised_count = count_arrays(carp_depleted_metasomatised,depth_normal,depth_fields,j)
		carp_fertile_lherz_count = count_arrays(carp_fertile_lherz,depth_normal,depth_fields,j)
		carp_melt_metas_count = count_arrays(carp_melt_metas,depth_normal,depth_fields,j)
		carp_unclass_count = count_arrays(carp_unclass,depth_normal,depth_fields,j)

		for i in range(0,len_tot):
			if (depth_normal[i] < depth_fields[j]) and (depth_normal[i] >= depth_fields[j-1]):
				tot_num_depth = tot_num_depth + 1

		depth_count = carp_melt_metas_count + carp_fertile_lherz_count +\
		 carp_depleted_metasomatised_count + carp_deplet_harz_count + carp_deplet_lherz_count

		carp_deplet_harz_pct = pct_count(carp_deplet_harz_count,tot_num_depth)
		carp_deplet_lherz_pct = pct_count(carp_deplet_lherz_count,tot_num_depth)
		carp_depleted_metasomatised_pct = pct_count(carp_depleted_metasomatised_count,tot_num_depth)
		carp_fertile_lherz_pct = pct_count(carp_fertile_lherz_count,tot_num_depth)
		carp_melt_metas_pct = pct_count(carp_melt_metas_count,tot_num_depth)
		carp_unclass_pct = pct_count(carp_unclass_count,tot_num_depth)

		total = carp_melt_metas_pct + carp_fertile_lherz_pct + carp_depleted_metasomatised_pct +\
		carp_deplet_harz_pct + carp_deplet_lherz_pct

		deplet_harz_pct_list.append(carp_deplet_harz_pct)
		deplet_lherz_pct_list.append(carp_deplet_lherz_pct)
		deplet_metas_pct_list.append(carp_depleted_metasomatised_pct)
		fertile_pct_list.append(carp_fertile_lherz_pct)
		melt_pct_metas.append(carp_melt_metas_pct)
		unclass_pct.append(carp_unclass_pct)
		total_list.append(tot_num_depth)
		total_classified.append(depth_count)

	#making plot_ranges

	last_box = np.array(deplet_harz_pct_list) + np.array(deplet_lherz_pct_list) +\
	np.array(deplet_metas_pct_list) + np.array(fertile_pct_list) +\
	np.array(melt_pct_metas)
	fourth_box = np.array(deplet_harz_pct_list) + np.array(deplet_lherz_pct_list) +\
	np.array(deplet_metas_pct_list) + np.array(fertile_pct_list)
	third_box = np.array(deplet_harz_pct_list) + np.array(deplet_lherz_pct_list) + \
	np.array(deplet_metas_pct_list)
	second_box = np.array(deplet_harz_pct_list) + np.array(deplet_lherz_pct_list)
	first_box = np.array(deplet_harz_pct_list)

	deplet_harz_plot = []
	deplet_lherz_plot = []
	deplet_metas_plot = []
	fertile_plot = []
	melt_plot = []
	depth_plot = []

	for i in range(0,len(deplet_harz_pct_list)):
		for k in range(0,2):
			deplet_harz_plot.append(first_box[i])
			deplet_lherz_plot.append(second_box[i])
			deplet_metas_plot.append(third_box[i])
			fertile_plot.append(fourth_box[i])
			melt_plot.append(last_box[i])

			if i == 0:
				if k == 0:
					depth_plot.append(depth_fields[i])
			elif i == len(deplet_harz_pct_list)-1:
				depth_plot.append(depth_fields[i])
				if k == 1:
					depth_plot.append(depth_fields[i] + (depth_interval * 1e3))
			else:
				depth_plot.append(depth_fields[i])

	zeros = np.zeros(len(depth_plot))
	depth_plot = np.array(depth_plot)
	fig1 = plt.figure(figsize = (20,7))
	ax1 = plt.subplot2grid((16,16),(0,0), rowspan = 16,colspan = 2, fig = fig1)


	ax1.fill_betweenx(depth_plot / 1e3,melt_plot,zeros,color = '#ad0f1f',label = 'Melt Metasomatised')
	ax1.fill_betweenx(depth_plot / 1e3,fertile_plot,zeros,color = '#0b820f',label = 'Fertile')
	ax1.fill_betweenx(depth_plot / 1e3,deplet_metas_plot,zeros,color = '#0b3c82',label = 'Depleted Lherz. + Phlg. Metas.')
	ax1.fill_betweenx(depth_plot / 1e3,deplet_lherz_plot,zeros,color = '#dbc14e', label = 'Depleted Lherzolites')
	ax1.fill_betweenx(depth_plot / 1e3,deplet_harz_plot,zeros,color = '#ab963a',label = 'Depleted Harzburgite',hatch = 'xxx')
	for i in range(0,len(depth_fields)):
		ax1.axhline(depth_fields[i] / 1e3, linewidth = 0.75, alpha = 0.5, color = 'k',linestyle = '-.')
	ax1.invert_yaxis()
	ax1.legend(fontsize = 6, bbox_to_anchor=(0.05, 0.925), borderaxespad=0.1,framealpha = 1.0)
	ax1.set_xlim(0,100)
	ax1.set_ylim((np.amax(depth_fields) / 1e3) + 10, (np.amin(depth_fields) / 1e3)-10)
	ax1.set_facecolor('#f7f7f2')
	#ax1.set_ylim(max(depth_plot) + 20,40)
	ax1.set_xlabel('Percent')
	ax1.set_ylabel('Depth (km)')
	ax1.axhline(BDL_depth / 1e3,linewidth = 2,color = 'k',linestyle = ':',label = 'Y-edge')

	depth_second = []
	for i in range(1,len(depth_fields)):
		depth_second.append((depth_fields[i] + depth_fields[i-1])/2e3)
	ax2 = plt.subplot2grid((16,16),(0,2), rowspan = 16,colspan = 1, fig = fig1)
	ax2.plot(total_classified,depth_second,label = 'Classified', color = '#b02f1e')
	ax2.plot(total_list,depth_second, linestyle = '--',dashes=(5, 12), label = 'Analysed', color = 'k')
	ax2.set_ylim((np.amax(depth_fields) / 1e3)+10,(np.amin(depth_fields) / 1e3)-10)
	ax2.set_facecolor('#f7f7f2')
	#ax2.set_ylim(max(depth_plot) + 20,40)
	for i in range(0,len(depth_fields)):
		ax2.axhline((depth_fields[i] / 1e3), linewidth = 0.75, alpha = 0.5, color = 'k',linestyle = '-.')
	ax2.fill_betweenx(depth_second,total_list,total_classified,color = 'k',alpha = 0.1)
	ax2.legend(fontsize = 6)
	ax2.set_xlabel('No.Samples - Log')
	ax2.grid(alpha = 0.6)
	#ax2.set_title('CARP - analysed samples,\n total = ' + str(len(T_Ni)),fontsize = 4)
	ax2.set_yticklabels([])
	ax2.axhline(BDL_depth / 1e3,linewidth = 2,color = 'k',linestyle = ':')
	ax2.set_xscale('log')
	ax2.set_xlim((1,10000))
	ax2.set_xticks([1,10,100,1000,1e4])
	ax2.set_xticklabels([0,1,2,3,4])
	ax2.grid(which = 'minor')

	low_cr_pct_list = []
	lowca_pct_list = []
	ca_harz_pct_list = []
	lherz_pct_list = []
	wehr_pct_list = []
	unclas_pct_list = []

	for j in range(1,len(depth_fields)):
		lowcr_count = 0
		lowca_count = 0
		harz_count = 0
		lherz_count = 0
		wehr_count = 0
		unclass_count = 0
		for i in range(0,len(depth_normal)):
			if (depth_normal[i] < depth_fields[j]) and (depth_normal[i] >= depth_fields[j-1]):
				if cacr_class[i] == 'Low-Cr Peridotite':
					lowcr_count = lowcr_count + 1
				elif cacr_class[i] == 'Low-Ca Harzburgite':
					lowca_count = lowca_count + 1
				elif cacr_class[i] == 'Ca-Harzburgite':
					harz_count = harz_count + 1
				elif cacr_class[i] == 'Lherzolite':
					lherz_count = lherz_count + 1
				elif cacr_class[i] == 'Wehrlite':
					wehr_count = wehr_count + 1
				else:
					unclass_count = unclass_count + 1

		tot_count = lowcr_count + lowca_count + harz_count +\
		lherz_count + wehr_count + unclass_count

		if tot_count != 0:

			low_cr_pct_list.append((lowcr_count / tot_count) * 1e2)
			lowca_pct_list.append((lowca_count / tot_count) * 1e2)
			ca_harz_pct_list.append((harz_count / tot_count) * 1e2)
			lherz_pct_list.append((lherz_count / tot_count) * 1e2)
			wehr_pct_list.append((wehr_count / tot_count) * 1e2)
			unclas_pct_list.append((unclass_count / tot_count) * 1e2)
		else:
			low_cr_pct_list.append(0)
			lowca_pct_list.append(0)
			ca_harz_pct_list.append(0)
			lherz_pct_list.append(0)
			wehr_pct_list.append(0)
			unclas_pct_list.append(0)

	last_box = np.array(lowca_pct_list) + np.array(ca_harz_pct_list) +\
	np.array(lherz_pct_list) + np.array(low_cr_pct_list) +\
	np.array(wehr_pct_list)
	fourth_box = np.array(lowca_pct_list) + np.array(ca_harz_pct_list) +\
	np.array(lherz_pct_list) + np.array(low_cr_pct_list)
	third_box = np.array(lowca_pct_list) + np.array(ca_harz_pct_list) +\
	np.array(lherz_pct_list)
	second_box = np.array(lowca_pct_list) + np.array(ca_harz_pct_list)
	first_box = np.array(lowca_pct_list)


	lowca_plot = []
	ca_harz_plot = []
	lherz_plot = []
	lowcr_plot = []
	wehr_plot = []
	depth_plot = []

	for i in range(0,len(lowca_pct_list)):
		for k in range(0,2):
			lowca_plot.append(first_box[i])
			ca_harz_plot.append(second_box[i])
			lherz_plot.append(third_box[i])
			lowcr_plot.append(fourth_box[i])
			wehr_plot.append(last_box[i])
			if i == 0:
				if k == 0:
					depth_plot.append(depth_fields[i])
			elif i == len(deplet_harz_pct_list)-1:
				depth_plot.append(depth_fields[i])
				if k == 1:
					depth_plot.append(depth_fields[i] + depth_interval)
			else:
				depth_plot.append(depth_fields[i])

	depth_plot = np.array(depth_plot)

	zeros = np.zeros(len(depth_plot))
	ax3 = plt.subplot2grid((16,16),(0,3), rowspan = 16,colspan = 2, fig = fig1)
	ax3.fill_betweenx(depth_plot / 1e3, wehr_plot,zeros,color = '#b06f90',label = 'Wehrlite')
	ax3.fill_betweenx(depth_plot / 1e3, lowcr_plot,zeros,color = '#d9c1cb',label = 'Low-Cr')
	ax3.fill_betweenx(depth_plot / 1e3, lherz_plot,zeros,color = '#339e51',label = 'Lherzolite')
	ax3.fill_betweenx(depth_plot / 1e3, ca_harz_plot,zeros,color = '#ef2fa1', label = 'Ca Harzburgite')
	ax3.fill_betweenx(depth_plot / 1e3, lowca_plot,zeros,color = '#8b1f8f',label = 'Low-Ca Harzburgite',hatch = 'xxx')
	for i in range(0,len(depth_fields)):
		ax3.axhline(depth_fields[i] / 1e3, linewidth = 0.75, alpha = 0.5, color = 'k',linestyle = '-.')
	ax3.invert_yaxis()
	ax3.set_facecolor('#f7f7f2')
	ax3.legend(fontsize = 8, bbox_to_anchor=(0.075, 0.925), borderaxespad=0.1,framealpha = 1.0)
	ax3.set_xlim(0,100)
	ax3.set_xlabel('Percent')
	ax3.set_ylim((np.amax(depth_fields) / 1e3)+10,(np.amin(depth_fields)/1e3)-10)
	#ax3.set_ylim(max(depth_plot) + 20,40)
	ax3.set_yticklabels([])
	ax3.axhline(BDL_depth / 1e3,linewidth = 2,color = 'k',linestyle = ':')

	g10_list = []
	g10d_list = []
	g9_list = []
	g12_list = []
	g1_list = []
	g11_list = []
	g5_list = []
	g5d_list = []
	g4_list = []
	g4d_list = []
	g3_list = []
	g3d_list = []
	g0_list = []

	for j in range(1,len(depth_fields)):

		g10_count = 0
		g10d_count = 0
		g9_count = 0
		g12_count = 0
		g1_count = 0
		g11_count = 0
		g5_count = 0
		g5d_count = 0
		g4_count = 0
		g4d_count = 0
		g3_count = 0
		g3d_count = 0
		g0_count = 0
		tot_num_depth = 0

		for i in range(0,len(depth_normal)):
			if (depth_normal[i] < depth_fields[j]) and (depth_normal[i] >= depth_fields[j-1]):
				if g_class[i] == 'G10':
					g10_count = g10_count + 1
				elif g_class[i] == 'G10D':
					g10d_count = g10d_count + 1
				elif g_class[i] == 'G9':
					g9_count = g9_count + 1
				elif g_class[i] == 'G12':
					g12_count = g12_count + 1
				elif g_class[i] == 'G1':
					g1_count = g1_count + 1
				elif g_class[i] == 'G11':
					g11_count = g11_count + 1
				elif g_class[i] == 'G5':
					g5_count = g5_count + 1
				elif g_class[i] == 'G5D':
					g5d_count = g5d_count + 1
				elif g_class[i] == 'G4':
					g4_count = g4_count + 1
				elif g_class[i] == 'G4D':
					g4d_count = g4d_count + 1
				elif g_class[i] == 'G3':
					g3_count = g3_count + 1
				elif g_class[i] == 'G3D':
					g3d_count = g3d_count + 1
				elif g_class[i] == 'G0':
					g0_count + g0_count + 1

		depth_count = g10_count + g10d_count + g9_count + g12_count +\
		g1_count + g11_count + g5_count + g5d_count + g4_count + g4d_count +\
		g3_count + g3d_count + g0_count

		g10_list.append(pct_count(g10_count,depth_count))
		g10d_list.append(pct_count(g10d_count,depth_count))
		g9_list.append(pct_count(g9_count,depth_count))
		g12_list.append(pct_count(g12_count,depth_count))
		g1_list.append(pct_count(g1_count,depth_count))
		g11_list.append(pct_count(g11_count,depth_count))
		g5_list.append(pct_count(g5_count,depth_count))
		g5d_list.append(pct_count(g5d_count,depth_count))
		g4_list.append(pct_count(g4_count,depth_count))
		g4d_list.append(pct_count(g4d_count,depth_count))
		g3_list.append(pct_count(g3_count,depth_count))
		g3d_list.append(pct_count(g3d_count,depth_count))
		g0_list.append(pct_count(g0_count,depth_count))

	ax4 = plt.subplot2grid((16,16),(0,5), rowspan = 16,colspan = 2, fig = fig1)

	list_lists = [np.array(g10_list),np.array(g10d_list),
	np.array(g9_list),np.array(g12_list),np.array(g1_list),
	np.array(g11_list),np.array(g5_list),np.array(g5d_list),
	np.array(g4_list),np.array(g4d_list),np.array(g3_list),
	np.array(g3d_list),np.array(g0_list)]
	labels_g = ['G10','G10D','G9','G12','G1','G11','G5','G5D','G4','G4D',
	'G3','G3D','G0']
	colors_g = ['#F18F01','#CB7A01', '#81A739','#79B791', '#048BA8', '#036277',
	'#FFDD33','#CCAA00','#D84727','#9B341C','#896a8a','#5C475C','#F7FFF7']
	boxes_g = []
	boxes_g_plot = []
	for i in range(0,len(list_lists)):
		boxes_g.append(sum(list_lists[:i+1]))
	for i in range(0,len(boxes_g)):
		boxes_g_plot_local = []
		for j in range(0,len(boxes_g[i])):
			for k in range(0,2):
				boxes_g_plot_local.append(boxes_g[i][j])
		boxes_g_plot.append(boxes_g_plot_local)

	boxes_g_plot = boxes_g_plot[::-1]
	labels_g = labels_g[::-1]
	colors_g = colors_g[::-1]
	for i in range(0,len(boxes_g_plot)):
		ax4.fill_betweenx(depth_plot / 1e3,boxes_g_plot[i],zeros,color = colors_g[i], label = labels_g[i])

	for i in range(0,len(depth_fields)):
		ax4.axhline(depth_fields[i] / 1e3, linewidth = 0.75, alpha = 0.5, color = 'k',linestyle = '-.')

	ax4.invert_yaxis()
	ax4.set_facecolor('#f7f7f2')
	ax4.legend(fontsize = 6, bbox_to_anchor=(0.05, 0.925), borderaxespad=0.075,framealpha = 1.0,ncol = 3)
	ax4.set_xlim(0,100)
	ax4.set_xlabel('Percent')
	ax4.set_ylim((np.amax(depth_fields) / 1e3)+10,(np.amin(depth_fields)/1e3)-10)
	#ax4.set_ylim(max(depth_plot) + 20,40)
	ax4.set_yticklabels([])
	ax4.axhline(BDL_depth/1e3,linewidth = 2,color = 'k',linestyle = ':')


	ax5 = plt.subplot2grid((16,16),(0,7), rowspan = 16,colspan = 5, fig = fig1)

	for i in range(0,len(geotherms)):
		ax5.plot(geotherms[i],depths[i] / 1e3,'--', linewidth = 0.75)

	d_g_p = np.array(d_g_p)
	z_1 = [2.50051235e-11, -9.09533105e-09,  1.34139387e-06, -1.03026414e-04,  4.40294443e-03, -1.03459682e-01,  4.27676616e+00]
	p_cr_depth = ((z_1[0] * (P_Cr * 10)**6.0) + (z_1[1] * (P_Cr * 10)**5.0) + (z_1[2] * (P_Cr * 10)**4.0) + (z_1[3] * (P_Cr * 10)**3.0) + (z_1[4] * (P_Cr * 10)**2.0) + (z_1[5] * (P_Cr * 10)) + z_1[6]) * (P_Cr * 10)
	d_g_depth = ((z_1[0] * d_g_p**6.0) + (z_1[1] * d_g_p**5.0) + (z_1[2] * d_g_p**4.0) + (z_1[3] * d_g_p**3.0) + (z_1[4] * d_g_p**2.0) + (z_1[5] * d_g_p) + z_1[6]) * d_g_p

	ax5.plot(d_g_t,d_g_depth,'-',color = '#a832a4')
	ax5.plot(T_chosen,depth_chosen/1e3,'--', color = 'k', linewidth = 2)

	ax5.set_facecolor('#f7f7f2')

	color_list_carp = ['#ab963a','#dbc14e','#0b3c82','#0b820f','#ad0f1f','k']
	carp_depleted_p = []
	carp_depleted_t = []
	carp_depleted_y = []

	for i in carp_depleted_harz:
		carp_depleted_p.append(p_cr_depth[i])
		carp_depleted_t.append(T_Ni[i])
	ax5.plot(carp_depleted_t,carp_depleted_p,'d', color = color_list_carp[0],
	markeredgewidth = 0.3,markeredgecolor = 'k', label = 'Depleted Harzburgite')

	carp_depleted_lherz_p = []
	carp_depleted_lherz_t = []
	carp_depleted_lherz_y = []
	for i in carp_depleted_lherz:
		carp_depleted_lherz_p.append(p_cr_depth[i])
		carp_depleted_lherz_t.append(T_Ni[i])
	ax5.plot(carp_depleted_lherz_t,carp_depleted_lherz_p,'s', color = color_list_carp[1],
	markeredgewidth = 0.3,markeredgecolor = 'k',label = 'Depleted Lherzolite')

	carp_met_depleted_p = []
	carp_met_depleted_t = []
	carp_met_depleted_y = []
	for i in carp_depleted_metasomatised:
		carp_met_depleted_p.append(p_cr_depth[i])
		carp_met_depleted_t.append(T_Ni[i])
	ax5.plot(carp_met_depleted_t,carp_met_depleted_p,'^', color = color_list_carp[2],
	markeredgewidth = 0.3,markeredgecolor = 'k',label = 'Depleted Lherzolite + Phlg. Metas.')

	carp_fertile_lherz_p = []
	carp_fertile_lherz_t = []
	carp_fertile_lherz_y = []
	for i in carp_fertile_lherz:
		carp_fertile_lherz_p.append(p_cr_depth[i])
		carp_fertile_lherz_t.append(T_Ni[i])
	ax5.plot(carp_fertile_lherz_t,carp_fertile_lherz_p,'*', color = color_list_carp[3],
	markeredgewidth = 0.3,markeredgecolor = 'k',label = 'Fertile')

	carp_melt_metas_p = []
	carp_melt_metas_t = []
	carp_melt_metas_y = []
	for i in carp_melt_metas:
		carp_melt_metas_p.append(p_cr_depth[i])
		carp_melt_metas_t.append(T_Ni[i])
	ax5.plot(carp_melt_metas_t,carp_melt_metas_p,'h', color = color_list_carp[4],
	markeredgewidth = 0.3,markeredgecolor = 'k',label = 'Melt Metasomatised')

	carp_unclass_p = []
	carp_unclass_t = []
	carp_unclass_y = []
	for i in carp_unclass:
		carp_unclass_p.append(p_cr_depth[i])
		carp_unclass_t.append(T_Ni[i])
	ax5.plot(carp_unclass_t,carp_unclass_p,'x', color = color_list_carp[5],label = 'Unclassified')

	ax5.axhline(BDL_depth/1e3,linewidth = 2,color = 'k',linestyle = ':')
	ax5.set_xlim((500,1500))
	ax5.set_ylim((np.amax(depth_fields)/1e3)+10,(np.amin(depth_fields)/1e3)-10)
	#ax5.set_ylim(max(depth_plot) + 20,40)
	for i in range(0,len(depth_fields)):
		ax5.axhline(depth_fields[i]/1e3, linewidth = 0.75, alpha = 0.5, color = 'k',linestyle = '-.')
	ax5.set_xlabel(r'$T_{Ni}({}^{\circ}C)$')
	ax5.set_yticklabels([])


	ax5.axvline(BDL_T,linestyle = '--',color = 'k',linewidth = 1)

	#ax5.legend(fontsize = 8, bbox_to_anchor=(0.25, 0.97), borderaxespad=0.1,framealpha = 1.0)
	ax5.legend(fontsize = 8,framealpha = 1.0)

	al_mean_list = []
	al_median_list = []

	for j in range(1,len(depth_fields)):
		al_local = []
		for i in range(0,len(depth_normal)):
			if (depth_normal[i] < depth_fields[j]) and (depth_normal[i] >= depth_fields[j-1]):
				al_local.append(AL[i])

		al_mean_list.append(np.mean(np.array(al_local)))
		al_median_list.append(np.median(np.array(al_local)))

	ax6 = plt.subplot2grid((16,16),(0,12), rowspan = 16,colspan = 1, fig = fig1)
	ax6.plot(al_mean_list,depth_second,label = 'Mean', color = '#b02f1e')
	ax6.plot(al_median_list,depth_second, linestyle = '--',dashes=(5, 12), label = 'Median', color = 'k')
	ax6.set_ylim((np.amax(depth_fields)/1e3)+10,(np.amin(depth_fields)/1e3)-10)
	ax6.set_facecolor('#f7f7f2')
	#ax6.set_ylim(max(depth_plot) + 20,40)
	for i in range(0,len(depth_fields)):
		ax6.axhline(depth_fields[i]/1e3, linewidth = 0.75, alpha = 0.5, color = 'k',linestyle = '-.')
	ax6.legend(fontsize = 6,framealpha = 1.0)
	ax6.set_xlim((0,3))
	ax6.axhline(BDL_depth/1e3,linewidth = 2,color = 'k',linestyle = ':')
	ax6.set_xlabel(r'WR $Al_2O_3$')
	ax6.set_yticklabels([])

	mgno_mean_list = []
	mgno_median_list = []

	for j in range(1,len(depth_fields)):
		mgno_local = []
		for i in range(0,len(depth_normal)):
			if (depth_normal[i] < depth_fields[j]) and (depth_normal[i] >= depth_fields[j-1]):
				mgno_local.append(MG[i])

		mgno_mean_list.append(np.mean(np.array(mgno_local)))
		mgno_median_list.append(np.median(np.array(mgno_local)))

	ax7 = plt.subplot2grid((16,16),(0,13), rowspan = 16,colspan = 1, fig = fig1)
	for item in [0.88,0.9,0.92,0.94]:
		ax7.axvline(item,linewidth = 0.5,color = 'k', alpha = 0.3)
	ax7.plot(mgno_mean_list,depth_second,label = 'Mean', color = '#b02f1e')
	ax7.plot(mgno_median_list,depth_second, linestyle = '--',dashes=(5, 12), label = 'Median', color = 'k')
	ax7.set_ylim((np.amax(depth_fields)/1e3)+10,(np.amin(depth_fields)/1e3)-10)
	ax7.set_facecolor('#f7f7f2')
	#ax7.set_ylim(max(depth_plot) + 20, 40)
	for i in range(0,len(depth_fields)):
		ax7.axhline(depth_fields[i]/1e3, linewidth = 0.75, alpha = 0.5, color = 'k',linestyle = '-.')
	ax7.axhline(BDL_depth / 1e3,linewidth = 2,color = 'k',linestyle = ':')

	ax7.legend(fontsize = 6,framealpha = 1.0)
	ax7.set_xlabel(r'#${Mg}^{ol}$')
	ax7.set_xlim((0.87,0.96))
	ax7.set_yticklabels([])
	ax7.set_xticks([0.88,0.9,0.92,0.94,0.96])
	ax7.set_xticklabels([88,90,92,94,96],fontsize = 7)

	if plot_type == 'show':
		plt.show()
	elif plot_type == 'save':
		if filename_save == None:
			save_str = os.path.join((os.getcwd(),'CARP_PLOT.png'),bbox_inches = "tight")
			plt.savefig(save_str, dpi = 300.0)
		else:
			try:
				plt.savefig(filename_save, dpi = 300.0,bbox_inches = "tight")
			except ValueError:
				print('Not a proper format for the filename_save')
	return plt


import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from functools import partial
import inspect
import warnings as w
import numbers
from pathlib import Path
Thermobar_dir=Path(__file__).parent
from Thermobar.import_export import *
from Thermobar.core import *

def convert_F2O_to_F_ppm(F2O_wt=None):
    """ Converts from F2O in wt% to F in ppm"""
    F2O_mass=53.99621
    F_mass=18.998403
    F_calc=2*F_mass*(F2O_wt/F2O_mass)
    return F_calc*10000

def convert_F_to_F2O(F_ppm=None):
    """ Converts from F in ppm to F2O in wt%"""
    F=F_ppm/10000
    F2O_mass=53.99621
    F_mass=18.998403
    F2O_calc=(F/F_mass)*0.5*F2O_mass
    return F2O_calc


def normalize_anhydrous_to_100_incF_mol_prop(liq_comps, F2O_content=0):
    '''
    Calculates normalized hydrous molar proportions following the normalization scheme
    of Giordano et al. (2008) in their excel spreadsheet.

    Parameters
    -------

    liq_comps: pandas.DataFrame
        liquid compositions with column headings SiO2_Liq, MgO_Liq etc.

    F2O_content: int, pd.series
        F2O content of the liquid (wt%), by default is set at zero.

    Returns
    -------
    pd.DataFrame
        column headings of the form Liq_mol_frac_hyd, ...
    '''


    if isinstance(F2O_content, pd.Series):
        F2O_content.fillna(0, inplace=True)
    liq_comps_C=liq_comps.copy()
    liq_comps_C['NiO_Liq']=0
    liq_comps_C['Cr2O3_Liq']=0
    liq_comps_C['CoO_Liq']=0
    liq_comps_C['CO2_Liq']=0

    liq_comps_sum=(liq_comps_C['SiO2_Liq']+liq_comps_C['TiO2_Liq']+
    liq_comps_C['Al2O3_Liq']+liq_comps_C['FeOt_Liq']+liq_comps_C['MnO_Liq']+liq_comps_C['MgO_Liq']+
    liq_comps_C['CaO_Liq']+liq_comps_C['Na2O_Liq']+liq_comps_C['K2O_Liq']+
    liq_comps_C['P2O5_Liq']+F2O_content)
    Norm_Factor=(100-liq_comps_C['H2O_Liq'])/(liq_comps_sum)


    if 'Sample_ID_Liq' in liq_comps_C.columns:
        liq_comps_C_nolabel= liq_comps_C.drop('Sample_ID_Liq', axis=1)
    else:
        liq_comps_C_nolabel=liq_comps_C
    liq_comps_c_N=liq_comps_C_nolabel.multiply(Norm_Factor, axis=0)

    liq_comps_c_N=liq_comps_C_nolabel.multiply(Norm_Factor, axis=0)
    # Don't normalize water
    liq_comps_c_N['H2O_Liq']=liq_comps_C['H2O_Liq']
    liq_comps_N_mol=calculate_hydrous_mol_proportions_liquid(liq_comps=liq_comps_c_N)

    liq_comps_N_mol['F2O_Liq_mol_prop_hyd']=F2O_content*Norm_Factor/37.9968
    liq_comps_N_mol['F2O_Liq_mol_prop_hyd'].fillna(0, inplace=True)
    mols_sum = 100/(liq_comps_N_mol.sum(axis='columns'))
    cat_frac_anhyd = liq_comps_N_mol.multiply(mols_sum, axis='rows')
    cat_frac_anhyd.columns = [str(col).replace('mol_prop', 'mol_frac')
                              for col in cat_frac_anhyd.columns]

    #print(cat_frac_anhyd)


    return cat_frac_anhyd  #mols_sum #liq_comps_N_mol['sum']



def calculate_viscosity_giordano_2008(liq_comps, T=None, T_K=None, H2O_Liq=None, F2O_content=0):
    '''
    Calculates viscosity parameters A, B, C from Giordano et al. 2008.
    If a temperature is supplied, calculates viscosity.

    Parameters
    -------

    liq_comps: pandas.DataFrame
        liquid compositions with column headings SiO2_Liq, MgO_Liq etc.

    F2O_content: int, pd.series
        F2O content of the liquid (wt%), by default is set at zero.

    Optional:

    T or T_K: int, flt, pd.Series
        Temperature in Kelvin.
        If specified, returns viscosity and log viscosity

    H2O_Liq: int, flt, pd.Series
        Water content in wt%, overwrites that in input spreadsheet

    Returns
    -------
    pd.DataFrame
        viscosity parameters, viscosity (if T supplied), as well user-entered
        liquid composition.
    '''
    if T_K is not None:
        T=T_K
    liq_comps_c=liq_comps.copy()
    if H2O_Liq is not None:
        liq_comps_c['H2O_Liq']=H2O_Liq
        print('Water content from input overridden')




    mol_percents=normalize_anhydrous_to_100_incF_mol_prop(liq_comps=liq_comps_c,
    F2O_content=F2O_content)

    V = mol_percents["H2O_Liq_mol_frac_hyd"] + mol_percents["F2O_Liq_mol_frac_hyd"]
    TA = mol_percents["TiO2_Liq_mol_frac_hyd"] + mol_percents["Al2O3_Liq_mol_frac_hyd"]
    FM = mol_percents["FeOt_Liq_mol_frac_hyd"] + mol_percents["MnO_Liq_mol_frac_hyd"] + mol_percents["MgO_Liq_mol_frac_hyd"]
    NK = mol_percents["Na2O_Liq_mol_frac_hyd"] + mol_percents["K2O_Liq_mol_frac_hyd"]

    b1 = 159.5600 * (mol_percents["SiO2_Liq_mol_frac_hyd"] + mol_percents["TiO2_Liq_mol_frac_hyd"])
    b2 = -173.34 * mol_percents["Al2O3_Liq_mol_frac_hyd"]
    b3 = 72.13 * (mol_percents["FeOt_Liq_mol_frac_hyd"] + mol_percents["MnO_Liq_mol_frac_hyd"] + mol_percents["P2O5_Liq_mol_frac_hyd"])
    b4 = 75.69 * mol_percents["MgO_Liq_mol_frac_hyd"]
    b5 = -38.98 * mol_percents["CaO_Liq_mol_frac_hyd"]
    b6 = -84.08 * (mol_percents["Na2O_Liq_mol_frac_hyd"] + V)
    b7 = 141.54 * (V + np.log(1 + mol_percents["H2O_Liq_mol_frac_hyd"]))
    b11 = -2.43 * ((mol_percents["SiO2_Liq_mol_frac_hyd"] + mol_percents["TiO2_Liq_mol_frac_hyd"]) * FM)
    b12 = (-0.91 * ((mol_percents["SiO2_Liq_mol_frac_hyd"] + TA + mol_percents["P2O5_Liq_mol_frac_hyd"])
                    * (NK + mol_percents["H2O_Liq_mol_frac_hyd"])))
    b13 = 17.62 * (mol_percents["Al2O3_Liq_mol_frac_hyd"] * NK)

    c1 = 2.75 * mol_percents["SiO2_Liq_mol_frac_hyd"]
    c2 = 15.72 * TA
    c3 = 8.32 * FM
    c4 = 10.20 * mol_percents["CaO_Liq_mol_frac_hyd"]
    c5 = -12.29 * NK
    c6 = -99.54 * np.log(1 + V)
    c11 = 0.30 * ((mol_percents["Al2O3_Liq_mol_frac_hyd"] + FM + mol_percents["CaO_Liq_mol_frac_hyd"] - mol_percents["P2O5_Liq_mol_frac_hyd"]) * (NK + V))

    df_B = pd.DataFrame(np.array([b1,b2,b3,b4,b5,b6,b7,b11,b12,b13]).T,
                        columns = ['b1','b2','b3','b4','b5','b6','b7','b11','b12','b13']
                       )
    df_C = pd.DataFrame(np.array([c1,c2,c3,c4,c5,c6,c11]).T,
                        columns = ['c1','c2','c3','c4','c5','c6','c11']
                       )

    df_params = pd.concat([df_B, df_C],axis = 'columns')





    A = -4.55
    B = df_params.loc[:,'b1':'b13'].sum(axis = 'columns')
    C = df_params.loc[:,'c1':'c11'].sum(axis = 'columns')


    liq_comps_c.insert(0, 'A', A)
    liq_comps_c.insert(1, 'B', B.values)
    liq_comps_c.insert(2, 'C', C.values)

    if T is not None:
        df_params['T_K'] = T
        logn_melt = A + (B / (df_params['T_K'] - C))
        n_melt=10**(logn_melt)
        df_params['logn_melt'] = logn_melt
        liq_comps_c.insert(0, 'T_K', T)
        liq_comps_c.insert(0, 'logn_melt', logn_melt.values)
        liq_comps_c.insert(0, 'n_melt', n_melt.values)
    combo=pd.concat([liq_comps_c, df_params], axis=1)

    return combo




import numpy as np
import os, sys
import matplotlib.pyplot as plt

def calculate_hasterok2011_geotherm(SHF, BDL_T, T_0, max_depth, moho, kinked, adiabat, **kwargs):

	'''

	A function to calculate the generalized steady-state geotherm for the
	continental lithosphere as taken from Hasterok & Chapman (2011).
	:cite: `hasterok2011`

	All the parameters used in constructing the geotherm are derived from the
	cited study. Generalized continental geotherm can be creating assuming
	26% of the heat generation occurs in the upper crustal layer.

	Parameters
	-------

	SHF: Surface Heat Flow value in mW/m^2

	BDL_T: is Temperature at the base of the depleted lithosphere in Celsius

	T_0: is Temperature at the surface in Celsius

	max_depth: Maximum depth the geotherm is going to be calculated for.

	moho: The depth of Mohorivicic discontinuity (i.e. crustal thickness)

	kinked: Boolean parameter to determine whether the conductive
	geotherm will be kinked parallel to D-G transition curve after the BDL.

	out_format: str
        Choose from:

        |  'Array' - A numpy array output
        |  'DataFrame' - Pandas dataframe output

	Returns
	-------
	Temperature (Kelvin), Depth (meters), Pressure (GPa), index point where the geotherm
	is kinked

	'''

	out_format = kwargs.pop('out_format', 'Array')

	if kinked == False:
		BDL_T = 0
	else:
		BDL_T = BDL_T
		d_g_t = [600,700,800,900,1000,1100,1200,1300,1400,1500,1600]
		d_g_p = [35,37.5,40,42.5,45,47.5,50,52.5,55,57.5,60]
		d_g_p = np.array(d_g_p) * 0.1
		d_g_t = np.array(d_g_t)
		d_g_slope = (np.array(d_g_t)[-1] - np.array(d_g_t)[0]) / (d_g_p[-1] - d_g_p[0])

	moho = moho * 1e3 #converting moho depth to meters
	T_0 = T_0 + 273.0 #converting celsius to kelvin
	max_depth = max_depth * 1e3 #converting max depth to meters.

	g = 9806.0
	rho_crust = 2850.0
	rho_mantle = 3340.0
	A_upper_crust = 1e-3 #mW/m^3
	A_lower_crust = 0.4e-3 #mW/m^3
	heat_prod_mantle = 4e-5 #mW/m^3

	interval = 1000.0 #Depth interval for calculations, in kms
	depth = np.arange(0,(max_depth) + interval, interval) #setting up the depth range in meters

	#Setting up the thermal conductivity parameters.
	k0_upper_1 = 1.496e3
	k0_upper_2 = 2.964e3
	k0_mid_1 = 1.733e3
	k0_mid_2 = 2.717e3
	k0_low_1 = 1.723e3
	k0_low_2 = 2.32e3
	k0_mantle_sp = 2.271e3
	k0_mantle_gt = 2.371e3
	k1_upper_1 = 398.84e3
	k1_upper_2 = -495.29e3
	k1_mid_1 = 194.59e3
	k1_mid_2 = -398.93e3
	k1_low_1 = 219.88e3
	k1_low_2 = -96.88e3
	k1_mantle_sp = 681.12e3
	k1_mantle_gt = 669.4e3
	k2_upper_1 = 4.573e-4
	k2_upper_2 = 0.866e-4
	k2_mid_1 = 2.906e-4
	k2_mid_2 = 0.032e-4
	k2_low_1 = 1.705e-4
	k2_low_2 = -0.981e-4
	k2_mantle_sp = -1.259e-4
	k2_mantle_gt = -1.288e-4
	k3_upper_1 = 0.0950
	k3_upper_2 = 0.0692
	k3_mid_1 = 0.0788
	k3_mid_2 = 0.0652
	k3_low_1 = 0.0520
	k3_low_2 = 0.0384
	k3_mantle_sp = 0.0399
	k3_mantle_gt = 0.0384

	p = np.zeros(len(depth)) #Setting up pressure array
	density_mantle = np.zeros(len(depth)) #setting up density array

	for i in range(1,len(depth)):
		if depth[i] <= moho:
			p[i] = interval * rho_crust * g
			density_mantle[i] = rho_crust
		else:
			p[i] = interval * rho_mantle * g
			density_mantle[i] = rho_mantle

	p = np.cumsum(p) / 1e12 #cumulative addition and converting to GPa

	t_criterion = 844.0 #Kelvin

	def calculate_k_st(k0,k1,k2,k3,Temp,P):

		k = (k0 + (k1 / Temp) + (k2 * Temp**2.0)) * (1 + (k3 * P))

		return k

	def interception(y1,y2):
		#Local function to calculate the interception point of arrays that have
		#used same temperature variation as paramaeter:(y1(T),y2(T))
		idx = np.argwhere(np.diff(np.sign(y1 - y2)) != 0)

		return idx

	#Setting up parameters as numpy arrays at length of d_z (layers)
	T = np.zeros(len(depth)) #Temperature in Kelvin
	q = np.zeros(len(depth)) #Heat flow
	A_list = np.zeros(len(depth)) #Heat generation
	k = np.zeros(len(depth)) #Thermal conductivity
	#Setting up the first layers
	T[0] = T_0 #Temperature at surface
	q[0] = SHF #Heat flow at surface
	k[0] = (k0_upper_1 + (k1_upper_1 / T[0]) + (k2_upper_1 * T[0]**2.0)) #Thermal
	#conductivity at surface, calculated via Hasterok (2010, PhD thesis).

	#Logical parameter that will be changed when sp-garnet transition
	#going to be intercepted.
	transition_sp_search = True

	hr = 16 * 1e3
	A_upper_crust = (SHF*0.26) / 16000.0 #26% of heat generation happens in first 16 km

	#Setting up for loop that will iterate for number of layers.
	for i in range(0,len(depth)):
		#If conditionals setting the heat production parameters based on given input.

		if depth[i] <= hr:
			A_list[i] = A_upper_crust
		elif (depth[i] > hr) and (depth[i] <= moho):
			A_list[i] = A_lower_crust
		elif (depth[i] > moho) and (depth[i] <= max_depth):
			A_list[i] = heat_prod_mantle

		if i != 0:

			#Solving the temperature with formula (?)
			T[i] = T[i-1] + ((q[i-1] * interval) / k[i-1]) - ((A_list[i-1] * interval**2.0) / (2.0 * k[i-1]))
			T_avg = (T[i-1] + T[i]) / 2.0 #Averaging T

			p_spinel = 1.4209 + np.exp((3.9073 * 1e-3 * T) - 6.8041)  #Spinel-Garnet transition line calculated with T
			#Taken from Hasterok & Chapman (2011)
			p_spinel = p_spinel[np.nonzero(T)] #Getting rid of zeros


			#Calculating T-dependent thermal conductivity
			#If conditionals for different empirical parameters for critical temperature (Hasterok, 2010)

			if T[i] <= t_criterion:
				if depth[i] <= hr:
					k[i] = calculate_k_st(k0_upper_1,k1_upper_1,k2_upper_1,k3_upper_1,T_avg,p[i])
				elif (depth[i] > hr) and (depth[i] <= moho):
					k[i] = calculate_k_st(k0_low_1,k1_low_1,k2_low_1,k3_low_1,T_avg,p[i])
				else:
					#In the mantle searching for sp-gt transition
					if transition_sp_search == True:
						idx_sp = interception(p_spinel,p[:len(p_spinel)])
						if idx_sp != None:
							#If producted geotherm intercepts P-T line of sp-gt transition
							#change logical parameter to False for changing to gt.
							transition_sp_search = False
							depth_spinel = depth[i] #Depth of transition
							p_spinel_trans = p[i]
							index_spinel = i
						k[i] = calculate_k_st(k0_mantle_sp,k1_mantle_sp,k2_mantle_sp,k3_mantle_sp,T_avg,p[i])
					else:
						k[i] = calculate_k_st(k0_mantle_gt,k1_mantle_gt,k2_mantle_gt,k3_mantle_gt,T_avg,p[i])
			else:
				#Same if tree for temperatures bigger than critical temperature : (t_criterion)
				if depth[i] <= hr:
					k[i] = calculate_k_st(k0_upper_2,k1_upper_2,k2_upper_2,k3_upper_2,T_avg,p[i])
				elif (depth[i] > hr) and (depth[i] <= moho):
					k[i] = calculate_k_st(k0_low_2,k1_low_2,k2_low_2,k3_low_2,T_avg,p[i])
				else:
					if transition_sp_search == True:
						idx_sp = interception(p_spinel,p[:len(p_spinel)])
						if idx_sp != None:
							transition_sp_search = False
							depth_spinel = depth[i]
							p_spinel_trans = p[i]
						k[i] = calculate_k_st(k0_mantle_sp,k1_mantle_sp,k2_mantle_sp,k3_mantle_sp,T_avg,p[i])
					else:
						k[i] = calculate_k_st(k0_mantle_gt,k1_mantle_gt,k2_mantle_gt,k3_mantle_gt,T_avg,p[i])

			q[i] = q[i-1] - (A_list[i-1] * interval)
			if depth[i] == moho:
				crust_production = SHF - q[i]
				moho_heat_flow = q[i]

	if kinked == True:

		idx_geotherm_nearest = (np.abs(T-BDL_T)).argmin()
		t_geotherm_inflict = T[idx_geotherm_nearest]
		p_geotherm_inflict = p[idx_geotherm_nearest]
		depth_geotherm_inflict = depth[idx_geotherm_nearest]

		kinked_geotherm_add = t_geotherm_inflict - (d_g_slope * p_geotherm_inflict)
		kinked_geotherm = kinked_geotherm_add + (d_g_slope * p)

		T[idx_geotherm_nearest:] = kinked_geotherm[idx_geotherm_nearest:]

		if adiabat == True:
			adiabat = False
			print('Both "kinked" and "adiabat" cannot be True. Turning adiabat==False.')

	else:

		idx_geotherm_nearest = 0

	if adiabat == True:

		T_C_Adiabat, T_K_Adiabat = T_Katsura_2022_Adiabat(p)

		idx_T = np.argwhere(np.diff(np.sign(T - T_K_Adiabat)) != 0)

		try:
			T[idx_T[0][0]:] = T_K_Adiabat[idx_T[0][0]:]
		except IndexError:
			pass


	if out_format == 'Array':

		return T, depth, p, idx_geotherm_nearest
		
	elif out_format == 'DataFrame':

		out = pd.DataFrame(
	            data={'T_K': T, 'Depth_Km': depth, 'Pressure_GPa': p})

		return out

def T_Katsura_2022_Adiabat(P_input):

	'''
	A function that calculates the mantle adiabat temperature for given pressure
	range. Taken from Katsura (2022).

	###Parameters###
	P_input: Pressure in GPa

	'''

	Depth = [50,70,90,100,120,140,160,180,200,220,240,260,280,300,320,340,360,380,
	400,410,410,420,440,460,480,500]
	P = [1.5,2.1,2.8,3.1,3.8,4.4,5.1,5.8,6.4,7.1,7.8,8.5,9.2,9.9,10.6,11.2,
	11.9,12.6,13.4,13.7,13.7,14.1,14.9,15.6,16.4,17.1]
	T = [1646,1657,1667,1672,1682,1691,1700,1709,1718,1726,1735,1743,1751,
	1759,1766,1774,1781,1788,1796,1799,1860,1863,1871,1878,1885,1892]

	Depth = np.array(Depth)
	T_C = np.array(T) - 273.15

	# T_C_out = np.interp(P_input[P_input_idx_1:P_input_idx_2], P, T_C)
	T_C_out = np.interp(P_input ,P ,T_C)
	T_K_out = T_C_out + 273.15

	return T_C_out, T_K_out

def invert_generalised_mantle_geotherm(P_sample, T_sample, std_P, std_T, SHF_start, SHF_end, SHF_increment, max_depth, plot_solution,**kwargs):

	'''
	A function to invert a generalized geotherm to the input thermobarometric
	data.

	###Parameters###

	P_sample: Pressure of thermobarometric solution in GPa.

	T_sample: Temperature of thermobarometric solution in Kelvin.

	std_P: Standard deviation of thermobarometric solution in GPa.

	std_T: Standard deviation of thermobarometric solution in K.

	SHF_start: Starting Surface Heat Flow value to search solution (mW/m^2).

	SHF_end: The Last Surface Heat Flow value to search solution (mW/m^2).

	SHF_increment: Solution search invrement in (mW/m^2).

	max_depth: Maximum depth of the geotherm calculation in km.

	kinked: Boolean parameter to determine whether geotherm is kinked at the
		Base of the Depelted lithosphere.

	BDL_T: Temperature at the Base of the Depleted Lithosphere in K.

	adiabat: Boolean parameter to determine whether geotherm is
	calculated with an adiabat after LAB.

	plot_solution: Boolean parameter to choose whether plot a small
	graph of the solution.

	'''

	kinked = kwargs.pop('kinked', False)
	BDL_T = kwargs.pop('BDL_T', 180)
	adiabat = kwargs.pop('adiabat', False)

	#Setting up initial geotherm to calculate P equivalents.
	T, depth, P, idx_geotherm_nearest = calculate_hasterok2011_geotherm(SHF = SHF_start,
	 BDL_T = BDL_T+273, T_0 = 0, max_depth = max_depth, moho = 38,
	  kinked = kinked, adiabat = adiabat)

	depth_xen = []
	index_depth_xen = []
	t_exclude = []

	for i in range(0,len(P_sample)):
		if P_sample[i] < np.amax(P):
			idx = (np.abs(P-P_sample[i])).argmin()
			index_depth_xen.append(idx)
		else:
			t_exclude.append(i)

	t_exclude = t_exclude[::-1]
	T_xen_plot = np.asarray(T_sample)
	p_xen_plot = np.asarray(P_sample)

	for i in t_exclude:
		T_xen_plot = np.delete(T_xen_plot,i)
		p_xen_plot = np.delete(p_xen_plot,i)

	if type(std_T) is not list:
		temp_err = np.ones(len(T_sample)) * std_T
	elif type(std_T) is list:
		temp_err = np.array(std_T)

	if type(std_T) is not list:
		pres_err = np.ones(len(P_sample)) * std_P
	elif type(std_T) is list:
		pres_err = np.array(std_P)

	heat_flow_search = np.arange(SHF_start, SHF_end, SHF_increment)

	misfit_list = []

	for i in range(0,len(heat_flow_search)):

		T, depth, p, idx_geotherm_nearest = calculate_hasterok2011_geotherm(SHF = heat_flow_search[i],
		 BDL_T = BDL_T+273, T_0 = 0, max_depth = max_depth, moho = 38,
		  kinked = kinked, adiabat = adiabat)

		P_equivalent = []
		T_equivalent = []
		for i in index_depth_xen:
			P_equivalent.append(p[i])
			T_equivalent.append(T[i])

		P_equivalent = np.asarray(P_equivalent)
		T_equivalent = np.asarray(T_equivalent)

		RMS = ((1.0/float(len(T_sample))) * np.sum((((T_sample-T_equivalent)**2.0)) / (std_T**2.0))) ** 0.5
		misfit_list.append(RMS)

	index_rms_misfit = misfit_list.index(min(misfit_list))
	shf_solution = heat_flow_search[index_rms_misfit]

	T, depth, p, idx_geotherm_nearest = calculate_hasterok2011_geotherm(SHF = shf_solution,
	 BDL_T = BDL_T+273, T_0 = 0, max_depth = max_depth, moho = 38,
	  kinked = kinked, adiabat = adiabat)

	if plot_solution == True:
		plt.figure(figsize = (5,5))
		ax = plt.subplot(111)
		ax.plot(heat_flow_search, misfit_list, '-', color = 'k')
		ax.plot(shf_solution, misfit_list[index_rms_misfit], 'o', color = 'g', label = 'Best Solution')
		ax.set_xlabel('Surface Heat Flow ($mW/m^2$)')
		ax.set_ylabel('RMS misfit of T (Thermometry - Geotherm)')
		ax.grid()
		ax.legend()

		plt.show()

	return shf_solution, T, depth, p, misfit_list
